﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

namespace HeuristicLab.Analysis.FitnessLandscape {
  [Item("Adaptive Walk Calculator", "Calculates characteristics from an adaptive walk.")]
  [StorableClass]
  public class AdaptiveWalkCalculator : NamedItem, ICharacteristicCalculator {
    [Storable]
    private AdaptiveWalk walker;

    [StorableConstructor]
    private AdaptiveWalkCalculator(bool deserializing) : base(deserializing) { }
    private AdaptiveWalkCalculator(AdaptiveWalkCalculator original, Cloner cloner)
      : base(original, cloner) {
      walker = cloner.Clone(original.walker);
    }
    public AdaptiveWalkCalculator() {
      Name = ItemName;
      Description = ItemDescription;
      walker = new AdaptiveWalk();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new AdaptiveWalkCalculator(this, cloner);
    }

    public IEnumerable<string> Characteristics {
      get {
        return new[] { "AutoCorrelation1", "CorrelationLength", "InformationContent",
        "PartialInformationContent", "DensityBasinInformation", "InformationStability", 
        "Diversity", "Regularity", "TotalEntropy", "PeakInformationContent",
        "PeakDensityBasinInformation" };
      }
    }

    public bool CanCalculate(IProblem problem) {
      return problem is ISingleObjectiveHeuristicOptimizationProblem
        && problem.Operators.Any(x => x is IManipulator);
    }

    public IEnumerable<KeyValuePair<string, IItem>> Calculate(IProblem problem, IEnumerable<string> characteristics = null) {
      var soProb = (ISingleObjectiveHeuristicOptimizationProblem)problem;
      walker.Problem = soProb;
      walker.Prepare(true);
      using (var waitHandle = new AutoResetEvent(false)) {
        EventHandler evHandle = (sender, e) => {
          if (walker.ExecutionState == ExecutionState.Paused
          || walker.ExecutionState == ExecutionState.Stopped) waitHandle.Set();
        };
        walker.ExecutionStateChanged += evHandle;
        walker.Start();
        waitHandle.WaitOne();
        walker.ExecutionStateChanged -= evHandle;
      }
      var props = (characteristics ?? Characteristics).ToList();
      foreach (var p in props) {
        yield return new KeyValuePair<string, IItem>(p, walker.Results[p].Value);
      }
      walker.Prepare(true);
    }
  }
}
